@@ -8,16 +8,18 @@ from pathlib import Path
 import typer
 from pydantic import BaseModel
 
+from pipelime.choixe import XConfig
+from pipelime.choixe.visitors.processor_ui import ProcessorUi
 from pipelime.cli.parser import CLIParsingError, parse_pipelime_cli
 from pipelime.cli.subcommands import SubCommands as subc
 from pipelime.cli.utils import (
     PipelimeSymbolsHelper,
+    PipelimeUserAppDir,
     print_command_op_stage_info,
     print_commands_ops_stages_list,
 )
 
 if t.TYPE_CHECKING:
-    from pipelime.choixe import XConfig
     from pipelime.piper.checkpoint import Checkpoint
 
 
@@ -34,6 +36,7 @@ class PlCliOptions(BaseModel):
     command_outputs: t.Optional[Path]
     verbose: int
     dry_run: bool
+    no_ui: bool
     command: str
     command_args: t.List[str]
     pipelime_tmp: t.Optional[str]
@@ -117,6 +120,37 @@ def _deep_update_fn(to_be_updated: "XConfig", data: "XConfig", verbose: bool):
     return to_be_updated
 
 
+def _process_cfg(
+    cfg: "XConfig",
+    ctx: t.Optional["XConfig"],
+    run_all: t.Optional[bool],
+    ask_missing_vars: bool = False,
+    add_user_defined_vars: bool = False,
+) -> t.List["XConfig"]:
+    processor = ProcessorUi(
+        context=ctx,
+        cwd=cfg.get_cwd(),
+        allow_branching=True if (run_all is not None and run_all) else False,
+        ask_missing_vars=ask_missing_vars,
+    )
+    result = cfg.parse().accept(processor)
+
+    for res in result:
+        assert isinstance(res, t.Dict)
+
+    result = t.cast(t.List[t.Dict[str, t.Any]], result)
+
+    if add_user_defined_vars:
+        # add user defined variables to the result (it can be useful
+        # when we are processing the context multiple times)
+        for res in result:
+            res.update(processor._user_defined_vars)
+
+    cfgs = [XConfig(data=x, cwd=cfg.get_cwd(), schema=cfg.get_schema()) for x in result]
+
+    return cfgs
+
+
 def _process_cfg_or_die(
     cfg: "XConfig",
     ctx: t.Optional["XConfig"],
@@ -126,6 +160,8 @@ def _process_cfg_or_die(
     exit_on_error: bool,
     verbose: bool,
     print_all: bool,
+    ask_missing_vars: bool = False,
+    add_user_defined_vars: bool = False,
 ) -> t.List["XConfig"]:
     from pipelime.choixe.visitors.processor import ChoixeProcessingError
     from pipelime.cli.pretty_print import print_error, print_info, print_warning
@@ -134,10 +170,12 @@ def _process_cfg_or_die(
         print_info(f"> Processing {cfg_name}...")
 
     try:
-        effective_configs = (
-            [cfg.process(ctx)]
-            if run_all is not None and not run_all
-            else cfg.process_all(ctx)
+        effective_configs = _process_cfg(
+            cfg,
+            ctx,
+            run_all,
+            ask_missing_vars,
+            add_user_defined_vars,
         )
     except ChoixeProcessingError as e:
         if exit_on_error:
@@ -188,6 +226,8 @@ def _process_all(
     run_all: t.Optional[bool],
     exit_on_error: bool,
     verbose: int,
+    ask_missing_vars: bool = False,
+    add_user_defined_vars: bool = False,
 ):
     from pipelime.choixe import XConfig
     from pipelime.cli.pretty_print import print_info
@@ -206,6 +246,8 @@ def _process_all(
             exit_on_error,
             verbose > 1,
             verbose > 3,
+            ask_missing_vars,
+            add_user_defined_vars,
         )
         for c in base_cfg
         if c.to_dict()
@@ -238,6 +280,8 @@ def _process_all(
         exit_on_error,
         verbose > 1,
         verbose > 3,
+        ask_missing_vars,
+        add_user_defined_vars,
     )
 
 
@@ -363,6 +407,11 @@ def pl_main(
         count=True,
     ),
     dry_run: bool = typer.Option(False, "--dry-run", "-d", help="Dry run."),
+    no_ui: bool = typer.Option(
+        False,
+        "--no-ui",
+        help="Disable user interactions.",
+    ),
     command: str = typer.Argument(
         "",
         show_default=False,
@@ -442,12 +491,6 @@ def pl_main(
                 print(ctx.get_help())
         except ValueError:
             pass
-    elif command in subc.WIZARD[0] or any([w in command_args for w in subc.WIZARD[0]]):
-        from pipelime.cli.wizard import Wizard
-
-        if command and command not in subc.WIZARD[0]:
-            Wizard.model_cfg_wizard(command)
-        Wizard.model_cfg_wizard(command_args[0])
     elif command in subc.LIST[0]:
         print_commands_ops_stages_list(
             verbose > 0,
@@ -485,8 +528,8 @@ def pl_main(
             recursive=verbose > 1,
         )
     elif command:
-        from pipelime.piper.checkpoint import LocalCheckpoint
         from pipelime.choixe.utils.io import PipelimeTmp
+        from pipelime.piper.checkpoint import LocalCheckpoint
 
         # context auto-load
         if config and not context and ctx_autoload:
@@ -507,27 +550,33 @@ def pl_main(
             command_outputs=command_outputs,
             verbose=verbose,
             dry_run=dry_run,
+            no_ui=no_ui,
             command=command,
             command_args=command_args,
             pipelime_tmp=None,
         )
 
-        if checkpoint:
-            # create a new checkpoint
-            if checkpoint.exists():
-                if not checkpoint.is_dir():
-                    raise ValueError(
-                        f"Checkpoint folder `{checkpoint}` exists but is not a folder."
-                    )
-                if any(checkpoint.iterdir()):
-                    raise ValueError(f"Checkpoint folder `{checkpoint}` is not empty.")
-            plopts.pipelime_tmp = PipelimeTmp.make_session_dir().as_posix()
-            ckpt = LocalCheckpoint(folder=checkpoint)
-            ckpt.write_data(PlCliOptions._namespace, "", plopts.purged_dict())
+        if checkpoint is None:
+            checkpoint = PipelimeUserAppDir.temporary_checkpoint_path()
+            is_temp_checkpoint = True
         else:
-            ckpt = None
+            is_temp_checkpoint = False
 
-        run_with_checkpoint(cli_opts=plopts, checkpoint=ckpt)
+        # create a new checkpoint
+        if checkpoint.exists():
+            if not checkpoint.is_dir():
+                raise ValueError(
+                    f"Checkpoint folder `{checkpoint}` exists but is not a folder."
+                )
+            if any(checkpoint.iterdir()):
+                raise ValueError(f"Checkpoint folder `{checkpoint}` is not empty.")
+        plopts.pipelime_tmp = PipelimeTmp.make_session_dir().as_posix()
+        ckpt = LocalCheckpoint(folder=checkpoint)
+        ckpt.write_data(PlCliOptions._namespace, "", plopts.purged_dict())
+
+        run_with_checkpoint(
+            cli_opts=plopts, checkpoint=ckpt, is_temp_checkpoint=is_temp_checkpoint
+        )
     else:
         from pipelime.cli.pretty_print import print_error
 
@@ -535,18 +584,21 @@ def pl_main(
         raise typer.Exit(1)
 
 
-def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoint"]):
+def run_with_checkpoint(
+    cli_opts: PlCliOptions, checkpoint: "Checkpoint", is_temp_checkpoint: bool = False
+):
+    from loguru import logger
+
     import pipelime.choixe.utils.io as choixe_io
     from pipelime.choixe import XConfig
     from pipelime.cli.pretty_print import print_error, print_info, print_warning
-    from loguru import logger
 
     PipelimeSymbolsHelper.set_extra_modules(cli_opts.extra_modules)
 
     if cli_opts.pipelime_tmp:
         pltmp = Path(cli_opts.pipelime_tmp)
         if pltmp.is_dir():
-            logger.debug(f"Restoring pipelime temp folder to `{pltmp}`")
+            logger.debug(f"Pipelime temp folder: `{pltmp}`")
             choixe_io.PipelimeTmp.SESSION_TMP_DIR = pltmp
 
     if cli_opts.verbose > 0:
@@ -640,6 +692,8 @@ def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoi
                 True,
                 cli_opts.verbose > 2,
                 cli_opts.verbose > 3,
+                ask_missing_vars=not cli_opts.no_ui,
+                add_user_defined_vars=True,
             )
             partial_ctx_for_ctx = _ctx_for_ctx_update(ctx_for_ctx, new_ctxs)
 
@@ -658,6 +712,8 @@ def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoi
                 True,
                 cli_opts.verbose > 2,
                 cli_opts.verbose > 3,
+                ask_missing_vars=False,
+                add_user_defined_vars=False,
             )
             ctx_for_ctx = _ctx_for_ctx_update(ctx_for_ctx, new_ctxs)
 
@@ -714,36 +770,14 @@ def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoi
                 cli_opts.run_all,
                 False,
                 cli_opts.verbose > 2,
+                ask_missing_vars=False,  # do not allow missing vars in audit
+                add_user_defined_vars=False,
             )
         except ChoixeProcessingError as e:
-            # from rich.prompt import Confirm, Prompt
-
-            # from pipelime.cli.wizard import Wizard
-
             print_warning("Some variables are not defined in the context.")
             print_error(f"Invalid configuration! {e}")
             raise typer.Exit(1)
 
-            ### SKIP FOR NOW
-            # if not Confirm.ask(
-            #     "Do you want to create a new context?", default=True
-            # ):
-            #     print_error(f"Invalid configuration! {e}")
-            #     raise typer.Exit(1)
-        #
-        # print_info("\n📝 Please enter a value for each variable")
-        # new_ctx = Wizard.context_wizard(inspect_info.variables, effective_ctx)
-        #
-        # print_info("Processing configuration and context...", end="")
-        # effective_configs = _process_cfg_or_die(
-        #     base_cfg, new_ctx, run_all, output
-        # )
-        # print_info(" OK")
-        #
-        # outfile = Prompt.ask("\n💾 Write to (leave empty to skip)")
-        # if outfile:
-        #     new_ctx.save_to(Path(outfile).with_suffix(".yaml"))
-
         cfg_size = len(effective_configs)
         pls = "s" if cfg_size != 1 else ""
         print_info(f"🎉 Configuration successfully processed ({cfg_size} variant{pls}).")
@@ -766,6 +800,8 @@ def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoi
                 cli_opts.run_all,
                 True,
                 cli_opts.verbose,
+                ask_missing_vars=not cli_opts.no_ui,
+                add_user_defined_vars=False,
             )
 
         cmd_name = cli_opts.command
@@ -791,11 +827,13 @@ def run_with_checkpoint(cli_opts: PlCliOptions, checkpoint: t.Optional["Checkpoi
             run_command(
                 cmd_name,
                 cfg_dict,
-                cli_opts.verbose,
-                cli_opts.dry_run,
-                cli_opts.keep_tmp,
-                cli_opts.command_outputs,
-                checkpoint,
+                verbose=cli_opts.verbose,
+                dry_run=cli_opts.dry_run,
+                no_ui=cli_opts.no_ui,
+                keep_tmp=cli_opts.keep_tmp,
+                command_outputs=cli_opts.command_outputs,
+                checkpoint=checkpoint,
+                is_temp_checkpoint=is_temp_checkpoint,
             )
 
 
@@ -804,9 +842,11 @@ def run_command(
     cmd_args: t.Mapping,
     verbose: int,
     dry_run: bool,
+    no_ui: bool,
     keep_tmp: bool,
     command_outputs: t.Optional[Path],
-    checkpoint: t.Optional["Checkpoint"],
+    checkpoint: "Checkpoint",
+    is_temp_checkpoint: bool = False,
 ):
     """
     Run a pipelime command.
@@ -825,15 +865,31 @@ def run_command(
         time_to_str,
     )
     from pipelime.commands import TempCommand
+    from pipelime.piper.checkpoint import LocalCheckpoint
+
+    tui_ckpt_ns = checkpoint.get_namespace("__tui_data__")
 
     try:
         cmd_cls = get_pipelime_command_cls(command)
     except ValueError:
         raise typer.Exit(1)
 
-    if is_tui_needed(cmd_cls, cmd_args):
-        app = TuiApp(cmd_cls, cmd_args)
+    if is_temp_checkpoint and cmd_cls.save_to_default_checkpoint():
+        checkpoint = LocalCheckpoint(
+            folder=PipelimeUserAppDir.store_temporary_checkpoint()
+        )
+        tui_ckpt_ns = checkpoint.get_namespace("__tui_data__")
+
+    # if we are resuming, check if we need to show the tui
+    show_tui = tui_ckpt_ns.read_data("show", None)
+    if show_tui is None:
+        show_tui = is_tui_needed(cmd_cls, cmd_args)
+        tui_ckpt_ns.write_data("show", show_tui)
+
+    if show_tui and not no_ui:
+        app = TuiApp(cmd_cls, tui_ckpt_ns.read_data("cmd_args", cmd_args))
         cmd_args = t.cast(t.Mapping, app.run())
+        tui_ckpt_ns.write_data("cmd_args", cmd_args)
 
     if verbose > 2:
         print_info(f"\nCreating command `{command}` with options:")
@@ -846,6 +902,11 @@ def run_command(
             ckpt_ns = checkpoint.get_namespace(command)
             cmd_obj = cmd_cls.init_from_checkpoint(ckpt_ns, **cmd_args)
             cmd_obj._checkpoint = ckpt_ns
+            # NB: you may tempted to do now:
+            #     tui_ckpt_ns.write_data("show", False)
+            # however, the run may fail due to incorrect arguments
+            # so let's show the tui again if it was needed in the first place
+
     except ValidationError as e:
         show_field_alias_valerr(e)
         raise e
