@@ -1,5 +1,6 @@
 from pipelime.commands.general import (
     TimeItCommand,
+    StageTimingCommand,
     PipeCommand,
     CloneCommand,
     ConcatCommand,
@@ -8,10 +9,13 @@ from pipelime.commands.general import (
     RemoveRemoteCommand,
     ValidateCommand,
     MapCommand,
+    MapIfCommand,
     SortCommand,
     FilterCommand,
     SliceCommand,
     SetMetadataCommand,
+    FilterDuplicatesCommand,
+    CopySharedItemsCommand,
 )
 from pipelime.commands.shell import ShellCommand
 from pipelime.commands.split_ops import (
@@ -22,3 +26,4 @@ from pipelime.commands.split_ops import (
 from pipelime.commands.toy_dataset import ToyDatasetCommand
 from pipelime.commands.piper import RunCommand, DrawCommand, WatchCommand
 from pipelime.commands.tempman import TempCommand
+from pipelime.commands.resume import ResumeCommand
