@@ -380,6 +380,7 @@ class Item(t.Generic[T], metaclass=ItemFactory):
         class MyOtherItem(Item):
             @deferred_classattr
             def default_concrete(cls):
+                return MyOtherItemConcrete
     """
 
     _data_cache: t.Optional[T]
@@ -661,6 +662,12 @@ class Item(t.Generic[T], metaclass=ItemFactory):
             )
             if data_source is not None:
                 self._add_data_source(data_source)
+                if (
+                    self.cache_data is None
+                    and Item.is_cache_enabled(self.__class__) is False
+                    or self.cache_data is False
+                ):
+                    self._data_cache = None
 
     def remove_data_source(
         self: DerivedItemTp, *sources: _item_data_source
