@@ -254,9 +254,7 @@ def command(__func=None, *, title: t.Optional[str] = None, **__config_kwargs):
                 func(*fargs[0], *fargs[1], *fargs[2], **fargs[3])
 
         # override base docstring with a custom description
-        _FnModel.__doc__ = (
-            f"Autogenerated Pipelime command from `{func.__module__}.{func.__name__}`."
-        )
+        _FnModel.__doc__ = f"Autogenerated Pipelime command from `{func.__module__}.{func.__qualname__}`."
 
         # create the pipelime command class
         fmodel = type(
@@ -288,13 +286,19 @@ def command(__func=None, *, title: t.Optional[str] = None, **__config_kwargs):
                     return inspect.Parameter.empty
             return value
 
-        fmodel.__signature__ = fsig.replace(
-            parameters=[
-                p.replace(default=_unwrap_default(p.default))
-                for p in fsig_params.values()
-            ],
-            return_annotation=inspect.Signature.empty,
-        )
+        try:
+            fmodel.__signature__ = fsig.replace(
+                parameters=[
+                    p.replace(default=_unwrap_default(p.default))
+                    for p in fsig_params.values()
+                ],
+                return_annotation=inspect.Signature.empty,
+            )
+        except ValueError as e:
+            raise TypeError(
+                "Invalid command signature for @command "
+                f"`{func.__module__}.{func.__qualname__}`: {str(e)}"
+            ) from e
         return fmodel
 
     if __func is None:
@@ -339,9 +343,11 @@ class PipelimeCommand(
 
     Extra class definition kwargs:
         force_gc (bool): force garbage collection after the command is executed.
+        no_default_checkpoint (bool): do not save the command in the default checkpoint.
     """
 
     _force_gc: t.ClassVar[bool] = False
+    _no_default_checkpoint: t.ClassVar[bool] = False
     _track_callback: t.ClassVar[t.Optional[TrackCallback]] = None
 
     _piper: PiperInfo = PrivateAttr(default_factory=PiperInfo)  # type: ignore
@@ -361,8 +367,15 @@ class PipelimeCommand(
         return _LazyCommand
 
     @classmethod
-    def __init_subclass__(cls, force_gc: bool = False, **kwargs):
+    def __init_subclass__(
+        cls, force_gc: bool = False, no_default_checkpoint: bool = False, **kwargs
+    ):
         cls._force_gc = force_gc
+        cls._no_default_checkpoint = no_default_checkpoint
+
+    @classmethod
+    def save_to_default_checkpoint(cls) -> bool:
+        return not cls._no_default_checkpoint
 
     @classmethod
     def init_from_checkpoint(cls, checkpoint: CheckpointNamespace, /, **data):
